@@ -3,7 +3,7 @@
 #' @description
 #' Results of the AOP project are spatially aggregated on a hexagonal grid based
 #' on the global H3 index at resolution 8, with a size of 357 meters (short
-#' diagonal) and an area of 0.74 km2. More infor about H3 at
+#' diagonal) and an area of 0.74 km2. More information about H3 at
 #' \url{https://h3geo.org/docs/core-library/restable/}.  See documentation
 #' 'Details' for the data dictionary.
 #'
@@ -32,6 +32,9 @@
 #'}
 read_grid <- function(city, showProgress = TRUE){
 
+  # checks
+  if(! is.logical(showProgress) ){stop("The 'showProgress' argument must either be TRUE or FALSE")}
+
   # Get metadata with data url addresses
   temp_meta <- select_metadata(t="grid", c=city)
 
